import time

from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver import Keys, ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from tqdm import tqdm


OPTIONS_VALUES = {
    "select_option_11": "Los últimos 30 días",
    "select_option_21": "Este trimestre",
    "select_option_23": "El último trimestre"
}


def handle_open_url(pbar: tqdm, driver: WebDriver, looker_studio_url: str, description: str, update=True) -> None:
    """
       Abre url y actualiza la barra de progreso.

       Parámetros:
       ----------
       pbar : tqdm.std.tqdm
           Una instancia de la barra de progreso de `tqdm` que se utiliza para mostrar el progreso en la consola.

       driver : selenium.webdriver.chrome.webdriver.WebDriver
           La instancia del controlador de Selenium WebDriver que controla el navegador.

       looker_studio_url : str
           Una cadena de texto que sera la url a la que accederemos en el navegador.

       description : str
           Una cadena de texto que describe la acción actual que se está realizando. Esta descripción se muestra en la barra de progreso.

       Retorno:
       -------
       None
           Este método no retorna ningún valor.
    """
    pbar.set_description(description)
    driver.get(looker_studio_url)
    time.sleep(2)
    if update:
        pbar.update(1)


def handle_find_element_and_enter(pbar: tqdm, driver: WebDriver, description: str, element: str, value: str = None, ttl: int = 10) -> None:
    """
       Encuentra un elemento web por su XPath, interactúa con él y actualiza la barra de progreso.

       Este método busca un elemento en la página web utilizando Selenium, envía un valor de entrada si se proporciona,
       simula un evento de pulsación de tecla (ENTER), y luego espera un tiempo antes de continuar, actualizando la barra de progreso.

       Parámetros:
       ----------
       pbar : tqdm.std.tqdm
           Una instancia de la barra de progreso de `tqdm` que se utiliza para mostrar el progreso en la consola.

       driver : selenium.webdriver.chrome.webdriver.WebDriver
           La instancia del controlador de Selenium WebDriver que controla el navegador.

       description : str
           Una cadena de texto que describe la acción actual que se está realizando. Esta descripción se muestra en la barra de progreso.

       element : str
           Un selector XPath que identifica el elemento web en la página con el que se quiere interactuar.

       value : str, opcional
           El valor que se enviará al elemento web encontrado. Si es `None`, no se envía ningún valor de entrada (por defecto es `None`).

       ttl : int, opcional
           El tiempo en segundos que el script esperará después de interactuar con el elemento web antes de proceder. El valor predeterminado es 10 segundos.

       Retorno:
       -------
       None
           Este método no retorna ningún valor.
       """
    pbar.set_description(description)
    element_found = driver.find_element(By.XPATH, element)
    if value is not None:
        element_found.send_keys(value)
    element_found.send_keys(Keys.ENTER)
    time.sleep(ttl)
    pbar.update(1)


def handle_scroll_or_move(pbar: tqdm, driver: WebDriver, xpath: str, description: str,
                          action_type: str = "scroll") -> None:
    """
    Desplaza o mueve el cursor a un elemento especificado en la página y actualiza la barra de progreso.

    Parámetros:
    ----------
    pbar: tqdm
        Un objeto de tipo `tqdm` que representa la barra de progreso.

    driver: WebDriver
        Una instancia del navegador ChromeDriver.

    xpath: str
        El XPath del elemento al cual se desea desplazar o mover.

    description: str
        Una descripción del paso actual que se mostrará en la barra de progreso.

    action_type: str, opcional
        El tipo de acción a realizar: "scroll" para desplazamiento o "move" para mover el cursor. Por defecto es "scroll".

    Retorno:
    -------
    None
        Esta función no retorna ningún valor.
    """
    pbar.set_description(description)
    element = driver.find_element(By.XPATH, xpath)
    actions = ActionChains(driver)

    if action_type == "scroll":
        actions.scroll_to_element(element).perform()
    elif action_type == "move":
        actions.move_to_element(element).perform()

    pbar.update(1)
    time.sleep(5)  # Añadir tiempo de espera si es necesario


def handle_open_menu_options(pbar: tqdm, driver: WebDriver, menu_xpath: str, button_index: int) -> None:
    """
    Abre el menú mas opciones seleccionando un botón específico basado en su índice.

    Parámetros:
    ----------
    pbar: tqdm
        Un objeto de tipo `tqdm` que representa la barra de progreso.

    driver: WebDriver
        Una instancia del navegador WebDriver.

    menu_xpath: str
        El XPath del botón del menú del gráfico.

    button_index: int
        El índice del botón en la lista de botones del menú.

    Retorno:
    -------
    None
        Esta función no retorna ningún valor.
    """
    pbar.set_description("Abriendo Opciones de la tabla")
    menu_buttons = driver.find_elements(By.XPATH, menu_xpath)
    if button_index < len(menu_buttons):
        menu_button = menu_buttons[button_index]
        menu_button.click()
        time.sleep(5)  # Esperar para asegurar que el menú se abre completamente
        pbar.update(1)

def handle_export_data(pbar: tqdm, driver: WebDriver, export_xpath: str, button_name: str) -> None:
    """
    Exporta los datos haciendo clic en el botón de exportación basado en el nombre accesible.

    Parámetros:
    ----------
    pbar: tqdm
        Un objeto de tipo `tqdm` que representa la barra de progreso.

    driver: WebDriver
        Una instancia del navegador WebDriver.

    export_xpath: str
        El XPath del botón de exportación.

    button_name: str
        El nombre accesible del botón de exportación.

    Retorno:
    -------
    None
        Esta función no retorna ningún valor.
    """
    pbar.set_description("Exportando los datos")
    export_buttons = driver.find_elements(By.XPATH, export_xpath)
    for export_btn in export_buttons:
        if export_btn.accessible_name == button_name:
            export_btn.click()
            time.sleep(5)  # Esperar para asegurar que la exportación se procesa
    pbar.update(1)


def confirm_export(pbar: tqdm, driver: WebDriver, confirm_xpath: str) -> None:
    """
    Confirma la exportación haciendo clic en el botón de confirmación.

    Parámetros:
    ----------
    pbar: tqdm
        Un objeto de tipo `tqdm` que representa la barra de progreso.

    driver: WebDriver
        Una instancia del navegador WebDriver.

    confirm_xpath: str
        El XPath del botón de confirmación de exportación.

    Retorno:
    -------
    None
        Esta función no retorna ningún valor.
    """
    pbar.set_description("Confirmando exportación")
    confirm_export_btn = driver.find_element(By.XPATH, confirm_xpath)
    confirm_export_btn.click()
    time.sleep(30)  # Esperar para asegurar que la confirmación se procesa
    pbar.update(1)


def handle_find_wait_and_click(driver: WebDriver, find_xpath: str) -> None:
    element = WebDriverWait(driver, 10).until(
        EC.element_to_be_clickable((By.XPATH, find_xpath))
    )
    element.click()

def handle_find_visibility_and_click(driver: WebDriver, find_xpath: str) -> None:
    element = WebDriverWait(driver, 10).until(
        EC.visibility_of_element_located((By.XPATH, find_xpath))
    )
    element.click()


def handle_filters(pbar: tqdm, driver: WebDriver, select_option="select_option_21"):
    pbar.set_description("Aplicando filtros")

    # class datepicker
    handle_find_wait_and_click(driver, '//control-layout-wrapper[@class="datepicker"]')

    # select selectedDateRangeOption
    handle_find_wait_and_click(driver, '//md-select[@ng-model="$ctrl.selectedDateRangeOption"]')

    # select contains text
    handle_find_visibility_and_click(driver, f'//md-option//div[contains(text(), "{OPTIONS_VALUES[select_option]}")]')

    # class md-raised
    handle_find_wait_and_click(driver, '//*[contains(@class, "md-raised")]')

    time.sleep(5)  # Esperar para asegurar que la confirmación se procesa
    pbar.update(1)



import shutil
import sys
from pathlib import Path
from typing import Callable

from selenium.webdriver.chrome.options import Options
from selenium.webdriver.remote.webdriver import WebDriver
from tqdm import tqdm

from vacolba_looker_data.utils.webdriver.configurations import init_webdriver
from vacolba_looker_data.utils.webdriver.methods import (
    handle_open_url,
    handle_find_element_and_enter,
    handle_scroll_or_move,
    handle_open_menu_options,
    handle_export_data,
    confirm_export, handle_filters
)


def is_cache_available(cache_dir: Path) -> bool:
    """Verifica si el directorio de caché tiene archivos."""
    return cache_dir.exists() and any(cache_dir.iterdir())


def setup_driver_with_cache(chrome_options_func: Callable[[str, Path], Options], download_dir: str, cache_dir: Path, pbar: tqdm) -> WebDriver:
    """Configura el driver con el directorio de caché proporcionado."""
    chrome_options = chrome_options_func(download_dir, cache_dir.resolve())
    return init_webdriver(pbar, chrome_options)


def authenticate_user(driver: WebDriver, pbar: tqdm, looker_studio_url: str, user: str, password: str):
    """Realiza el flujo de autenticación del usuario."""
    handle_open_url(pbar, driver, looker_studio_url, "Abriendo Looker Studio", False)
    # handle_find_element_and_enter(pbar, driver, "Entrar al Looker Studio", '//*[@type="button"]')
    handle_find_element_and_enter(pbar, driver, "Iniciando sesión", '//*[@type="email"]', user, ttl=5)
    handle_find_element_and_enter(pbar, driver, "Ingresando contraseña", '//*[@type="password"]', password)
    handle_find_element_and_enter(pbar, driver, "Entrar al Looker Studio", '//*[@type="button"]')


def copy_cache(src_dir: Path, dest_dir: Path):
    """Copia el contenido de la caché de origen a destino."""
    if src_dir.exists():
        shutil.copytree(src_dir, dest_dir, dirs_exist_ok=True)


def main_flow(pbar: tqdm, driver: WebDriver, button_index: int = 3):
    """Ejecuta el flujo principal después de la autenticación."""
    if sys.argv[1]:
        handle_filters(pbar, driver, sys.argv[1])
    handle_scroll_or_move(pbar, driver, '//*[contains(@class, "simple-table")]', "Desplazándose a la tabla", action_type="scroll")
    handle_scroll_or_move(pbar, driver, '//*[contains(@class, "headerCell")]', "Moviéndose al encabezado de la tabla", action_type="move")
    handle_open_menu_options(pbar, driver, '//*[contains(@class, "ng2-chart-menu-button")]', button_index)
    handle_export_data(pbar, driver, '//*[contains(@class, "mat-mdc-menu-item")]', 'Exportar')
    confirm_export(pbar, driver, '//*[contains(@class, "mat-mdc-raised-button")]')


from tqdm import tqdm


def execute_with_progress(pbar: tqdm, description: str, func: callable, *args, **kwargs):
    """
    Ejecuta una función, actualiza la barra de progreso y muestra la descripción.

    Args:
        pbar (tqdm): La barra de progreso que se está actualizando.
        description (str): Descripción para el estado actual.
        func (callable): La función a ejecutar.
        *args: Argumentos posicionales para la función.
        **kwargs: Argumentos nombrados para la función.

    Returns:
        Cualquier cosa que retorne la función 'func'.
    """
    pbar.set_description(description)
    result = func(*args, **kwargs)
    pbar.update(1)
    return result


import os
from pathlib import Path
from dotenv import load_dotenv
from tqdm import tqdm

from vacolba_looker_data.utils.looker_studio.methods import is_cache_available, authenticate_user, copy_cache, main_flow, \
    setup_driver_with_cache
from vacolba_looker_data.utils.webdriver.methods import handle_open_url
from vacolba_looker_data.utils.webdriver.configurations import set_chrome_options

# Cargar archivo .env desde una ruta absoluta
env_path = Path('.') / '.env'  # Esto intenta cargar el .env en el directorio de trabajo actual
load_dotenv(dotenv_path=env_path)

# Configurar las variables de entorno
LOOKER_STUDIO_URL = os.getenv('LOOKER_STUDIO_URL')
VACOLBA_USER = os.getenv('VACOLBA_USER')
VACOLBA_PASS = os.getenv('VACOLBA_PASS')
DOWNLOAD_DIRECTORY = os.getenv('DOWNLOAD_DIRECTORY', str(Path('files').resolve()))
CACHE_DIRECTORY = Path('cache').resolve()
TEMP_CACHE_DIRECTORY = Path('temp_cache').resolve()
BUTTON_INDEX = os.getenv('BUTTON_INDEX', 3)


def get_looker_data(pbar: tqdm, looker_url: str = LOOKER_STUDIO_URL):
    """Obtiene los datos de Looker Studio, gestionando la autenticación y la caché."""
    # Configuración del driver
    cache_directory = CACHE_DIRECTORY if is_cache_available(CACHE_DIRECTORY) else TEMP_CACHE_DIRECTORY
    driver = setup_driver_with_cache(set_chrome_options, DOWNLOAD_DIRECTORY, cache_directory, pbar)

    try:
        if not is_cache_available(CACHE_DIRECTORY):
            # Autenticación y copia de la caché al directorio permanente si es necesario
            authenticate_user(driver, pbar, looker_url, VACOLBA_USER, VACOLBA_PASS)
            driver.quit()  # Cerrar navegador para copiar caché
            copy_cache(TEMP_CACHE_DIRECTORY, CACHE_DIRECTORY)
            driver = setup_driver_with_cache(set_chrome_options, DOWNLOAD_DIRECTORY, CACHE_DIRECTORY, pbar)
        else:
            pbar.set_description("Autenticacion obtenida")
            pbar.update(3)

        # Ejecutar flujo principal después de la autenticación
        handle_open_url(pbar, driver, looker_url, "Abriendo Looker Studio")
        main_flow(pbar, driver, int(BUTTON_INDEX))

    except Exception as e:
        print(f"Error: {e}")

    finally:
        pbar.set_description("Cerrando navegador")
        driver.quit()



"""
Setup configuration file
"""
from setuptools import setup, find_packages

setup(
    name='get_looker_data',
    packages=find_packages(),
    package_data={'vacolba_looker_data': ['looker_studio/*', 'utils/*', 'utils/looker_studio/*', 'utils/webdriver/*']},
    include_package_data=True,
    version='1.0.8',
    description='General functions for the implementation of looker studio data.',
    authors=[
        {"name": "Rene Gonzalez Ramos", "email": "rgramos9310@gmail.com"}
    ],
    license="GPLv3",
    url="https://github.com/rgramos/vacolba-looker-data.git",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ],
    entry_points={
        "flask.commands": [
            'generate-deploy = vacolba_looker_data.deploy:generate'
        ]
    }
)



from pathlib import Path

from selenium.webdriver.chrome.options import Options
from selenium import webdriver
from selenium.common import WebDriverException
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.remote.webdriver import WebDriver
from tqdm import tqdm
from webdriver_manager.chrome import ChromeDriverManager

def set_chrome_options(download_directory: str, cache_directory: Path) -> Options:
    """
    Configura las opciones de ChromeDriver para la automatización del navegador.

    Parámetros:
    ----------
    download_directory: str
        La ruta al directorio donde se descargarán los archivos.

    Retorno:
    -------
    Options
        Un objeto de tipo `Options` configurado con las preferencias de descarga y otras opciones de Chrome.
    """
    chrome_options = Options()
    chrome_options.add_argument("--start-maximized")  # Iniciar navegador maximizado

    # Configurar las preferencias de descarga de Chrome
    prefs = {
        "download.default_directory": download_directory,  # Configurar directorio de descarga
        "download.prompt_for_download": False,  # Desactivar la solicitud de confirmación de descarga
        "download.directory_upgrade": True,  # Permitir la actualización del directorio de descarga
        "safebrowsing.enabled": True  # Activar navegación segura
    }
    chrome_options.add_experimental_option("prefs", prefs)

    cache_directory = Path(cache_directory)

    # Configurar el directorio de cache del navegador
    if cache_directory.exists():
        chrome_options.add_argument(f"user-data-dir={cache_directory}")
    else:
        cache_directory.mkdir(parents=True, exist_ok=True)
        chrome_options.add_argument(f"user-data-dir={cache_directory}")

    return chrome_options


def init_webdriver(pbar: tqdm, chrome_options: Options) -> WebDriver:
    """
    Inicializa el navegador ChromeDriver con las opciones especificadas.

    Parámetros:
    ----------
    pbar: tqdm
        Un objeto de tipo `tqdm` que representa la barra de progreso.

    chrome_options: Options
        Un objeto de tipo `Options` configurado con las preferencias del navegador Chrome.

    Retorno:
    -------
    webdriver.Chrome
        Una instancia de ChromeDriver inicializada con las opciones proporcionadas.
    """
    try:
        # Paso 1: Inicializar el navegador
        pbar.set_description("Inicializando navegador")
        driver = webdriver.Chrome(options=chrome_options)
        pbar.update(1)

    except WebDriverException:
        pbar.set_description("Instalando ChromeDriver")
        driver_service = Service(executable_path=ChromeDriverManager().install())
        driver = webdriver.Chrome(options=chrome_options, service=driver_service)
        pbar.update(1)

    return driver
